@@ -34,6 +34,7 @@ ALLOWED_BLOCKS = [
 	'Devanagari Extended',
 	'Myanmar Extended-B',
 	'Myanmar Extended-A',
+	'Myanmar Extended-C',
 ]
 
 files = [open (x, encoding='utf-8') for x in sys.argv[1:]]
@@ -95,6 +96,7 @@ categories = {
     'PLACEHOLDER',
     'DOTTEDCIRCLE',
     'RS',
+    'MPst',
     'Repha',
     'Ra',
     'CM',
@@ -168,8 +170,6 @@ category_map = {
   'Vowel'			: 'V',
   'Vowel_Dependent'		: 'M',
   'Vowel_Independent'		: 'V',
-  'Dotted_Circle'		: 'DOTTEDCIRCLE', # Ours, not Unicode's
-  'Ra'				: 'Ra', # Ours, not Unicode's
 }
 position_map = {
   'Not_Applicable'		: 'END',
@@ -240,6 +240,9 @@ category_overrides = {
   0x0953: 'SM',
   0x0954: 'SM',
 
+  # U+0A40 GURMUKHI VOWEL SIGN II may be preceded by U+0A02 GURMUKHI SIGN BINDI.
+  0x0A40: 'MPst',
+
   # The following act like consonants.
   0x0A72: 'C',
   0x0A73: 'C',
@@ -440,7 +443,7 @@ for k,new_cat in category_overrides.items():
   indic_data[k] = (new_cat, pos, unicode_data[2][k])
 
 # We only expect position for certain types
-positioned_categories = ('CM', 'SM', 'RS', 'H', 'M')
+positioned_categories = ('CM', 'SM', 'RS', 'H', 'M', 'MPst')
 for k, (cat, pos, block) in indic_data.items():
   if cat not in positioned_categories:
     pos = 'END'
@@ -450,11 +453,12 @@ for k, (cat, pos, block) in indic_data.items():
 
 # Keep in sync with CONSONANT_FLAGS in the shaper
 consonant_categories = ('C', 'CS', 'Ra','CM', 'V', 'PLACEHOLDER', 'DOTTEDCIRCLE')
+matra_categories = ('M', 'MPst')
 smvd_categories = ('SM', 'VD', 'A', 'Symbol')
 for k, (cat, pos, block) in indic_data.items():
   if cat in consonant_categories:
     pos = 'BASE_C'
-  elif cat == 'M':
+  elif cat in matra_categories:
     if block.startswith('Khmer') or block.startswith('Myanmar'):
       cat = position_to_category(pos)
     else:
@@ -634,7 +638,7 @@ for u in uu:
 	end = (end-1)//8*8 + 7
 
 	if start != last + 1:
-		if start - last <= 1+16*3:
+		if start - last <= 1+16*2:
 			print_block (None, last+1, start-1, indic_data)
 		else:
 			if last >= 0:
@@ -691,6 +695,6 @@ print ('#endif')
 print ()
 print ("/* == End of generated table == */")
 
-# Maintain at least 30% occupancy in the table */
-if occupancy < 30:
+# Maintain at least 50% occupancy in the table */
+if occupancy < 50:
 	raise Exception ("Table too sparse, please investigate: ", occupancy)
